package preferences;

/**
 * @author Vaidas Gasiunas
 *
 * Aspect, which translates preference field access to preference retrieval from a preference source
 */
public abstract aspect PreferenceProvider {
	
	/* Subaspect must specify a concrete preference source */
	abstract protected PreferenceSource getPreferenceSource();
	
	/* Additional filter to limit scope of the aspect. 
	 * 		"if(true)" for global effect 
	 * 		"within(mypackage.*)" to limit accesses from certain packages
	 */
	abstract protected pointcut filter();

	/* intercept setting property of type String */
	void around(String val) : set(String PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().put(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept setting property of type int */
	void around(int val) : set(int PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().putInt(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept setting property of type long */
	void around(long val) : set(long PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().putLong(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept setting property of type boolean */
	void around(boolean val) : set(boolean PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().putBoolean(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept setting property of type double */
	void around(double val) : set(double PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().putDouble(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept setting property of type float */
	void around(float val) : set(float PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().putFloat(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept getting property of type byte[] */
	void around(byte[] val) : set(byte[] PreferenceContainer+.*) && args(val) 
			&& !within(PreferenceContainer+) && filter() {
		getPreferenceSource().putByteArray(thisJoinPointStaticPart.getSignature(), val);		
	}
	
	/* intercept getting property of type String */
	String around() : get(String PreferenceContainer+.*) && filter() {
		return getPreferenceSource().get(thisJoinPointStaticPart.getSignature(), proceed());
	}
	
	/* intercept getting property of type int */
	int around() : get(int PreferenceContainer+.*) && filter() {
		return getPreferenceSource().getInt(thisJoinPointStaticPart.getSignature(), proceed());
	}
	
	/* intercept getting property of type long */
	long around() : get(long PreferenceContainer+.*) && filter() {
		return getPreferenceSource().getLong(thisJoinPointStaticPart.getSignature(), proceed());
	}
	
	/* intercept getting property of type boolean */
	boolean around() : get(boolean PreferenceContainer+.*) && filter() {
		return getPreferenceSource().getBoolean(thisJoinPointStaticPart.getSignature(), proceed());
	}
	
	/* intercept getting property of type double */
	double around() : get(double PreferenceContainer+.*) && filter() {
		return getPreferenceSource().getDouble(thisJoinPointStaticPart.getSignature(), proceed());
	}
	
	/* intercept getting property of type float */
	float around() : get(float PreferenceContainer+.*) && filter() {
		return getPreferenceSource().getFloat(thisJoinPointStaticPart.getSignature(), proceed());
	}
	
	/* intercept getting property of type byte[] */
	byte[] around() : get(byte[] PreferenceContainer+.*) && filter() {
		return getPreferenceSource().getByteArray(thisJoinPointStaticPart.getSignature(), proceed());
	}	
}
