package preferences;

import java.util.prefs.Preferences;

import org.aspectj.lang.Signature;

/**
 * @author Vaidas Gasiunas
 *
 * Abstract preference source implementation using Preferences of JSDK
 */
public abstract class StandardPreferenceSource implements PreferenceSource {
	
	abstract protected Preferences getPreferences(Signature fieldInfo);
	
	protected String makePropertyName(Signature fieldInfo) {
		return fieldInfo.getDeclaringTypeName() + "." + fieldInfo.getName();
	}
	
	public String get(Signature fieldInfo, String def) {
		return getPreferences(fieldInfo).get(makePropertyName(fieldInfo), def);
	}
	
	public int getInt(Signature fieldInfo, int def) {
		return getPreferences(fieldInfo).getInt(makePropertyName(fieldInfo), def);
	}
	
	public long getLong(Signature fieldInfo, long def) {
		return getPreferences(fieldInfo).getLong(makePropertyName(fieldInfo), def);
	}
	
	public boolean getBoolean(Signature fieldInfo, boolean def) {
		return getPreferences(fieldInfo).getBoolean(makePropertyName(fieldInfo), def);
	}
	
	public double getDouble(Signature fieldInfo, double def) {
		return getPreferences(fieldInfo).getDouble(makePropertyName(fieldInfo), def);
	}
	
	public float getFloat(Signature fieldInfo, float def) {
		return getPreferences(fieldInfo).getFloat(makePropertyName(fieldInfo), def);
	}
	
	public byte[] getByteArray(Signature fieldInfo, byte[] def) {
		return getPreferences(fieldInfo).getByteArray(makePropertyName(fieldInfo), def);
	}
	
	public void put(Signature fieldInfo, String val) {
		getPreferences(fieldInfo).put(makePropertyName(fieldInfo), val);
	}
	
	public void putInt(Signature fieldInfo, int val) {
		getPreferences(fieldInfo).putInt(makePropertyName(fieldInfo), val);
	}
	
	public void putLong(Signature fieldInfo, long val) {
		getPreferences(fieldInfo).putLong(makePropertyName(fieldInfo), val);
	}
	
	public void putBoolean(Signature fieldInfo, boolean val) {
		getPreferences(fieldInfo).putBoolean(makePropertyName(fieldInfo), val);
	}
	
	public void putDouble(Signature fieldInfo, double val) {
		getPreferences(fieldInfo).putDouble(makePropertyName(fieldInfo), val);
	}
	
	public void putFloat(Signature fieldInfo, float val) {
		getPreferences(fieldInfo).putFloat(makePropertyName(fieldInfo), val);
	}
	
	public void putByteArray(Signature fieldInfo, byte[] val) {
		getPreferences(fieldInfo).putByteArray(makePropertyName(fieldInfo), val);
	}
}
